/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef USE_GTKUI

#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <unistd.h>

#include <gtk/gtkwidget.h>
#include <gtk/gtkcontainer.h>
#include <gtk/gtkbutton.h>
#include <gtk/gtktextbuffer.h>
#include <gtk/gtktextview.h>
#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtkvbox.h>
#include <gtk/gtkstock.h>
#include <gtk/gtkwindow.h>

#include "intl.h"
#include "logging.h"
#include "x_alloc.h"

#include "ui.h"
#include "conf.h"
#include "gtk-ui.h"
#include "gtk-utils.h"
#include "gtk-log.h"

struct XePGTKLog_s
{
  GtkWidget *window;
  GtkTextView *textview;
  GtkTextBuffer *buffer;
  XetPan *xetpan;
};

/* Log Viewer */

static gboolean
GtkLog_delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data);

static void
GtkLog_destroy_cb(GtkWidget *widget, gpointer data);

static void
GtkLog_refresh_cb(GtkButton *button, gpointer axetpan);

/* Function definitions */

XePGTKLog *
a_GtkLog_new(XetPan *xetpan, GtkWidget *toplevel)
{
  GtkWidget *window;
  GtkWidget *box;
  GtkWidget *scrollw;
  GtkWidget *textview;
  GtkWidget *button;
  XePGTKLog *logview;

  logview = x_alloc(1, XePGTKLog);
  if (logview == NULL)
    return NULL;

  logview->xetpan = xetpan;

  window = a_GtkUtils_transient_new(toplevel,
				    _("XetPan - Log"),
				    400, 300);

  box = gtk_vbox_new(FALSE, 12);
  gtk_container_add(GTK_CONTAINER(window), box);

  scrollw = gtk_scrolled_window_new(NULL, NULL);
  gtk_container_set_border_width(GTK_CONTAINER(scrollw), 0);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw), 
                                 GTK_POLICY_AUTOMATIC,
                                 GTK_POLICY_AUTOMATIC);

  gtk_box_pack_start(GTK_BOX(box), scrollw, TRUE, TRUE, 0);

  textview = a_GtkUtils_text_view_new(NULL, -1, FALSE);
  gtk_container_add(GTK_CONTAINER(scrollw), textview);

  button = gtk_button_new_from_stock(GTK_STOCK_REFRESH);
  gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);

  logview->window = window;
  logview->textview = GTK_TEXT_VIEW(textview);
  logview->buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(textview));

  /* Signals */
  
  g_signal_connect(G_OBJECT(window), "destroy",
		   G_CALLBACK(GtkLog_destroy_cb), logview);

  g_signal_connect(G_OBJECT(window), "delete-event",
		   G_CALLBACK(GtkLog_delete_cb), logview);

  g_signal_connect(G_OBJECT(button), "clicked",
		   G_CALLBACK(GtkLog_refresh_cb), logview);

  gtk_widget_show_all(box);

  return logview;
}

static gboolean
GtkLog_delete_cb(GtkWidget *widget, GdkEvent *event, gpointer data)
{
  XePGTKLog *logview = (XePGTKLog *) data;
  gtk_widget_hide(widget);

  gtk_text_buffer_set_text(logview->buffer, "", -1);

  return TRUE;
}

static void
GtkLog_destroy_cb(GtkWidget *widget, gpointer data)
{
  XePGTKLog *logview = (XePGTKLog *) data;
  x_free(logview);
}

static void
GtkLog_refresh_cb(GtkButton *button, gpointer logview)
{
  a_GtkLog_show((XePGTKLog *) logview);
}

void
a_GtkLog_show(XePGTKLog *logview)
{
  XetPan *xetpan = logview->xetpan;
  char *logfile;

  int fd;

  logfile = a_Conf_create_filename(xetpan->config_path, XETPAN_LOG_FILE);

  fd = open(logfile, O_RDONLY);
  if (fd == -1)
    {
      gchar *err_msg;
      err_msg = g_strdup_printf(_("Error while opening log file '%s': %s"),
				logfile, strerror(errno));
      gtk_text_buffer_set_text(logview->buffer, err_msg, -1);
      g_free(err_msg);
    }
  else
    {
      struct stat stat_buf;
      char *maddr;
      GtkTextIter iter;
      GtkTextMark *mark;

      fstat(fd, &stat_buf);

      if (stat_buf.st_size)
	{
	  maddr = mmap(NULL, stat_buf.st_size, PROT_READ, MAP_SHARED, fd, 0);
	  gtk_text_buffer_set_text(logview->buffer, maddr, stat_buf.st_size);
	  munmap(maddr, stat_buf.st_size);
	}
      else
	{
	  gtk_text_buffer_set_text(logview->buffer, _("<Log empty>"), -1);
	}
      close(fd);

      gtk_text_buffer_get_end_iter(logview->buffer, &iter);
      mark = gtk_text_buffer_create_mark(logview->buffer, NULL, &iter, TRUE);
      gtk_text_view_scroll_to_mark(logview->textview, mark, 0.0,
				   FALSE, 0.0, 0.0);
    }
  gtk_window_present(GTK_WINDOW(logview->window));
  g_free(logfile);

  a_UI_notify_error_reset(logview->xetpan);
}
#endif
