/*
 * XetPan: a libEtPan! based Mail User Agent
 * Copyright (C) 2002-2003 Melvin Hadasht
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef GTK_MB_EDITOR_H
#define GTK_MB_EDITOR_H

#include <gtk/gtkwidget.h>

#include "mailbox-types.h"

typedef struct XePGtkMbEditor_s XePGtkMbEditor;

/*! Create a mailbox editor
 *  \param toplevel the toplevel window associated with the editor
 *  \return a mailbox editor handle
 */
XePGtkMbEditor *
a_GtkMbEditor_new(GtkWidget *toplevel);

/*! Free the ressources used by a mailbox editor and destroy it
 */
void
a_GtkMbEditor_free(XePGtkMbEditor *mbeditor);

/*! Show the mailbox editor
 *  \param mbeditor the mailbox editor to show
 */
void
a_GtkMbEditor_show(XePGtkMbEditor *mbeditor);

/*! Hide the mailbox editor
 *  \param mbeditor the mailbox editor to hide
 */
void
a_GtkMbEditor_hide(XePGtkMbEditor *mbeditor);


/*! Set the mailbox to edit
 *  \param mbeditor the mailbox editor
 *  \param mailbox the mailbox to edit
 */
void
a_GtkMbEditor_set_mailbox(XePGtkMbEditor *mbeditor, XePMailbox *mailbox);

/*! Get the edited mailbox
 *  \param mbeditor the mailbox editor
 */
XePMailbox *
a_GtkMbEditor_get_mailbox(XePGtkMbEditor *mbeditor);

/*! Set the callback function to call when the OK button is clicked
 *  \param mbeditor the mailbox editor
 *  \param user_cb the callback function to call
 *  \param user_data the second argument to the callback function user_cb
 */
void
a_GtkMbEditor_set_ok_callback(XePGtkMbEditor *mbeditor,
			      void (*user_cb)(XePGtkMbEditor *ambeditor,
					      void *auser_data),
			      void *user_data);

#endif
